@@ -1,28 +1,35 @@
 #!/usr/bin/env python3
 
 from .bernoulli_likelihood import BernoulliLikelihood
-from .gaussian_likelihood import FixedNoiseGaussianLikelihood, GaussianLikelihood, _GaussianLikelihoodBase
+from .beta_likelihood import BetaLikelihood
+from .gaussian_likelihood import (
+    DirichletClassificationLikelihood,
+    FixedNoiseGaussianLikelihood,
+    GaussianLikelihood,
+    _GaussianLikelihoodBase,
+)
+from .laplace_likelihood import LaplaceLikelihood
 from .likelihood import Likelihood, _OneDimensionalLikelihood
 from .likelihood_list import LikelihoodList
-from .multitask_gaussian_likelihood import (
-    MultitaskGaussianLikelihood,
-    MultitaskGaussianLikelihoodKronecker,
-    _MultitaskGaussianLikelihoodBase,
-)
+from .multitask_gaussian_likelihood import MultitaskGaussianLikelihood, _MultitaskGaussianLikelihoodBase
 from .noise_models import HeteroskedasticNoise
 from .softmax_likelihood import SoftmaxLikelihood
+from .student_t_likelihood import StudentTLikelihood
 
 __all__ = [
     "_GaussianLikelihoodBase",
     "_OneDimensionalLikelihood",
     "_MultitaskGaussianLikelihoodBase",
     "BernoulliLikelihood",
+    "BetaLikelihood",
+    "DirichletClassificationLikelihood",
     "FixedNoiseGaussianLikelihood",
     "GaussianLikelihood",
     "HeteroskedasticNoise",
+    "LaplaceLikelihood",
     "Likelihood",
     "LikelihoodList",
     "MultitaskGaussianLikelihood",
-    "MultitaskGaussianLikelihoodKronecker",
     "SoftmaxLikelihood",
+    "StudentTLikelihood",
 ]
